#include <FG02_R5700K.h>

//=====================================================================
//    Sample sketch for Fuel Gauge Leaf
//
//    (c) 2022  Nisshinbo Micro Devices Inc.
//    Released under the MIT license
//    https://opensource.org/licenses/MIT
//
//=====================================================================
#include <Wire.h>
#include <FG02_R5700K.h>

//I2C Address of R5700 is 0x32
#define PIN_STAT 4
#define PIN_VCE 5

#define POFF_VOLTAGE  3400 

FG02_R5700K fg;
bool isSetChgCmp;

/**********************************************
* Disable DC/DC Converter for 5V Output
**********************************************/
void disableVIN()
{
  digitalWrite(PIN_VCE, LOW);
}

/**********************************************
* Check Charge Complete
**********************************************/
bool checkChargeComplete()
{
  bool chgCmp = false;
  if (digitalRead(PIN_STAT) == LOW)
  {
    chgCmp = true;
  }
  return chgCmp;
}

/**********************************************
* Serial I/F
**********************************************/
void serialVoltage(unsigned int voltage)
{
  Serial.print("Volt[mV]: ");
  Serial.print(voltage);
  Serial.print(" , ");
}

void serialCurrent(int current)
{
  Serial.print("Cur[mA]: ");
  Serial.print(current);
  Serial.print(" , ");
}

void serialSoc(float soc)
{
  Serial.print("SSOC[%]: ");
  Serial.print(soc, 1);
  Serial.print(" , ");
}

void serialTemperature(float temp)
{
  Serial.print("Temp[degC]: ");
  Serial.print(temp, 1);
  Serial.print(" , ");
}

/**********************************************
* Main
**********************************************/
void setup()
{
  Serial.begin(9600);
  Wire.begin();

  pinMode(PIN_STAT, INPUT);
  pinMode(PIN_VCE, OUTPUT);
  digitalWrite(PIN_VCE, HIGH);
  // Disalbe DCDC for 5.0V power supply
  disableVIN();

  // Battery UR18650ZY(2450mAh)
  fg.beginUr18650zy();
  fg.setOffset(0x0c);
  fg.enableFg();

  isSetChgCmp = false;

  Serial.println("FG-BAT Leaf On!  ");

  delay(3000);
}

void loop() {
  unsigned int voltage;
  int current;
  float soc;
  float temp;
  bool isChgCmp;

  // Get Voltage [mV]
  voltage = fg.getVoltage();
  serialVoltage(voltage);
  delay(3000);

  // Get Current [mA]
  current = fg.getCurrent();
  serialCurrent(current);
  delay(3000);

  // Check Charge Complete
  soc = fg.getSoc();
  if (soc != 100){
    isChgCmp = checkChargeComplete();
    if (isChgCmp == true) {
      if (isSetChgCmp == false){
        fg.setChargeComplete();
        isSetChgCmp = true;
      }
    } else {
      isSetChgCmp = false;
    }
  } else {
    isSetChgCmp = false;
  }

  // Check Discharge Complete
  if ((soc == 0) && (voltage < POFF_VOLTAGE)) {
    // SOC is 0%
    // Implement Power Off Process here
    Serial.print("Detect PowerOff , ");
  }

  // Get SOC [%]
  soc = fg.getSoc();
  serialSoc(soc);
  delay(3000);

  // Get Temperature [degC]
  temp = fg.getTemperature();
  serialTemperature(temp);
  delay(3000);

  Serial.println("");
}
